package com.qmth.boot.tools.excel.model;

import com.qmth.boot.tools.excel.annotation.ExcelColumn;
import com.qmth.boot.tools.excel.convertor.ValueConvertor;
import com.qmth.boot.tools.excel.convertor.ValueConvertors;

import java.lang.reflect.Field;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

public class ObjectParam extends HashMap<String, FieldParam> {

    private static final long serialVersionUID = 3789838085423920607L;

    private static Map<Class<?>, ObjectParam> paramMap = new ConcurrentHashMap<>();

    public static ObjectParam get(Class<?> clazz) {
        ObjectParam param = paramMap.get(clazz);
        if (param == null) {
            param = paramMap.computeIfAbsent(clazz, ObjectParam::new);
        }
        return param;
    }

    private String[] columnNames;

    private ObjectParam(Class<?> objectType) {
        super();

        Field[] fields = objectType.getDeclaredFields();
        for (Field field : fields) {
            field.setAccessible(true);
            ExcelColumn annotation = field.getAnnotation(ExcelColumn.class);
            if (annotation != null) {
                if (!ValueConvertors.containsType(field.getType())) {
                    if (!annotation.convertor().equals(ValueConvertor.class)) {
                        try {
                            ValueConvertors.register(field.getType(), annotation.convertor().newInstance());
                        } catch (Exception e) {
                            throw new RuntimeException(
                                    "Unsupported convertor in @ExcelColumn: " + annotation.convertor());
                        }
                    } else {
                        throw new RuntimeException("Unsupported field type with @ExcelColumn: " + field.getType());
                    }
                }
                put(annotation.name(), new FieldParam(field, annotation));
            }
        }

        List<FieldParam> list = new ArrayList<>(values());
        list.sort(Comparator.comparingInt(p -> p.getAnnotation().index()));
        columnNames = list.stream().map(p -> p.getAnnotation().name()).toArray(String[]::new);
    }

    public String[] getColumnNames() {
        return columnNames;
    }
}
